@@ -39,7 +39,7 @@
 #define GEDIT_VIEW_FRAME_SEARCH_DIALOG_TIMEOUT (30*1000) /* 30 seconds */
 
 #define MIN_SEARCH_COMPLETION_KEY_LEN	3
-#define SEARCH_POPUP_OFFSET 12
+#define SEARCH_POPUP_MARGIN 12
 
 #define GEDIT_VIEW_FRAME_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GEDIT_TYPE_VIEW_FRAME, GeditViewFramePrivate))
 
@@ -72,7 +72,6 @@ struct _GeditViewFramePrivate
 	guint        search_flags;
 
 	GtkWidget   *slider;
-	GtkWidget   *search_widget;
 	GtkWidget   *search_entry;
 	GtkCssProvider *search_css;
 
@@ -106,7 +105,7 @@ typedef enum
 /* The search entry completion is shared among all the views */
 GtkListStore *search_completion_model = NULL;
 
-G_DEFINE_TYPE_WITH_CODE (GeditViewFrame, gedit_view_frame, GTK_TYPE_VBOX,
+G_DEFINE_TYPE_WITH_CODE (GeditViewFrame, gedit_view_frame, GTK_TYPE_BOX,
                          g_type_add_class_private (g_define_type_id, sizeof (GeditViewFrameClassPrivate)))
 
 static void
@@ -1080,7 +1079,7 @@ create_search_widget (GeditViewFrame *frame)
 	                  G_CALLBACK (search_widget_scroll_event),
 	                  frame);
 
-	hbox = gtk_hbox_new (FALSE, 3);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
 	gtk_widget_show (hbox);
 
 	gtk_container_add (GTK_CONTAINER (search_widget), hbox);
@@ -1237,8 +1236,6 @@ init_search_entry (GeditViewFrame *frame)
 		guint     old_find_flags = 0;
 		gint      sel_len = 0;
 
-		g_free (frame->priv->old_search_text);
-
 		old_find_text = gedit_document_get_search_text (GEDIT_DOCUMENT (buffer),
 		                                                &old_find_flags);
 
@@ -1260,6 +1257,7 @@ init_search_entry (GeditViewFrame *frame)
 		}
 		else if (old_find_text != NULL)
 		{
+			g_free (frame->priv->old_search_text);
 			frame->priv->old_search_text = old_find_text;
 			add_search_completion_entry (old_find_text);
 			g_signal_handler_block (frame->priv->search_entry,
@@ -1285,6 +1283,12 @@ start_interactive_search_real (GeditViewFrame *frame)
 	GtkTextIter iter;
 	GtkTextMark *mark;
 
+	if (gtk_bin_get_child (GTK_BIN (frame->priv->slider)) == NULL)
+	{
+		gtk_container_add (GTK_CONTAINER (frame->priv->slider),
+			           create_search_widget (frame));
+	}
+
 	if (gtk_widget_get_visible (frame->priv->slider))
 	{
 		if (frame->priv->search_mode != frame->priv->request_search_mode)
@@ -1419,13 +1423,15 @@ gedit_view_frame_init (GeditViewFrame *frame)
 {
 	GeditDocument *doc;
 	GtkWidget *sw;
-	GeditOverlayChildPosition position;
 
 	frame->priv = GEDIT_VIEW_FRAME_GET_PRIVATE (frame);
 
 	frame->priv->typeselect_flush_timeout = 0;
 	frame->priv->wrap_around = TRUE;
 
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (frame),
+	                                GTK_ORIENTATION_VERTICAL);
+
 	doc = gedit_document_new ();
 
 	_gedit_document_set_mount_operation_factory (doc,
@@ -1433,6 +1439,7 @@ gedit_view_frame_init (GeditViewFrame *frame)
 						     frame);
 
 	frame->priv->view = gedit_view_new (doc);
+	gtk_widget_set_vexpand (frame->priv->view, TRUE);
 	gtk_widget_show (frame->priv->view);
 
 	g_object_unref (doc);
@@ -1448,33 +1455,25 @@ gedit_view_frame_init (GeditViewFrame *frame)
 
 	gtk_widget_show (sw);
 
-	frame->priv->overlay = gedit_animated_overlay_new (sw, frame->priv->view);
+	frame->priv->overlay = gedit_animated_overlay_new ();
+	gtk_container_add (GTK_CONTAINER (frame->priv->overlay), sw);
 	gtk_widget_show (frame->priv->overlay);
 
 	gtk_box_pack_start (GTK_BOX (frame), frame->priv->overlay, TRUE, TRUE, 0);
 
 	/* Add slider */
-	if (gtk_widget_get_direction (GTK_WIDGET (frame)) == GTK_TEXT_DIR_RTL)
-	{
-		position = GEDIT_OVERLAY_CHILD_POSITION_NORTH_WEST;
-	}
-	else
-	{
-		position = GEDIT_OVERLAY_CHILD_POSITION_NORTH_EAST;
-	}
-
-	frame->priv->search_widget = create_search_widget (frame);
-	frame->priv->slider = gedit_floating_slider_new (frame->priv->search_widget);
+	frame->priv->slider = gedit_floating_slider_new ();
+	gtk_widget_set_halign (frame->priv->slider, GTK_ALIGN_END);
+	gtk_widget_set_valign (frame->priv->slider, GTK_ALIGN_START);
+	gtk_widget_set_margin_right (frame->priv->slider, SEARCH_POPUP_MARGIN);
 	g_object_set (G_OBJECT (frame->priv->slider),
-	              "position", position,
-	              "offset", SEARCH_POPUP_OFFSET,
 	              "easing", GEDIT_THEATRICS_CHOREOGRAPHER_EASING_EXPONENTIAL_IN_OUT,
 	              "blocking", GEDIT_THEATRICS_CHOREOGRAPHER_BLOCKING_DOWNSTAGE,
 	              "orientation", GTK_ORIENTATION_VERTICAL,
 	              NULL);
 
-	gedit_animated_overlay_add (GEDIT_ANIMATED_OVERLAY (frame->priv->overlay),
-	                            GEDIT_ANIMATABLE (frame->priv->slider));
+	gedit_animated_overlay_add_animated_overlay (GEDIT_ANIMATED_OVERLAY (frame->priv->overlay),
+	                                             GEDIT_ANIMATABLE (frame->priv->slider));
 }
 
 GeditViewFrame *
